package com.extensibility.xml.dt;

import com.extensibility.util.Debug;
import com.extensibility.util.StringUtilities;
import com.extensibility.xml.CMTReferent;
import com.extensibility.xml.DTDParser;
import com.extensibility.xml.DataTypeRef;
import com.extensibility.xml.DeclRef;
import com.extensibility.xml.ParserException;
import com.extensibility.xml.SchemaIntf;
import com.extensibility.xml.XMLStringUtilities;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;

public class DataType implements DataTypeIntf, Cloneable, CMTReferent, DeclRef.SourceIntf {
   protected DataTypeRef type;
   protected DataTypeRef baseType;
   protected Hashtable facets;
   SchemaIntf schema = null;
   ChangeListener changerType = new 1(this);
   ChangeListener changerBase = new 2(this);
   EventListenerList listenerList = new EventListenerList();

   public DataType(String var1, String var2) {
      this.setTypeName(var1);
      this.setBaseType(var2);
      this.facets = new Hashtable();
   }

   public DataType(String var1, String var2, Vector var3) {
      this.setTypeName(var1);
      this.setBaseType(var2);
      this.facets = new Hashtable();
      if (var3 != null) {
         for(int var4 = 0; var4 < var3.size(); ++var4) {
            Facet var5 = (Facet)var3.elementAt(var4);
            this.facets.put(var5.getName(), var5);
         }
      }

   }

   public void setTypeName(String var1) {
      DataTypeRef var2 = this.type;
      if (this.type != null) {
         if (this.type.isBound()) {
            this.type.release();
         }

         this.type.removeChangeListener(this.changerType);
      }

      if (var1 != null) {
         this.type = new DataTypeRef(var1);
         if (this.schema != null) {
            this.type.bind(this.schema, this);
         }

         this.type.addChangeListener(this.changerType);
      } else {
         this.type = null;
      }

   }

   public String getTypeName() {
      return this.type != null ? this.type.getName() : "";
   }

   public boolean isQualified() {
      String var1 = this.getSource();
      return var1.indexOf(58) >= 0;
   }

   public void setBaseType(String var1) {
      DataTypeRef var2 = this.baseType;
      if (this.baseType != null) {
         if (this.baseType.isBound()) {
            this.baseType.release();
         }

         this.baseType.removeChangeListener(this.changerBase);
      }

      if (var1 != null) {
         this.baseType = new DataTypeRef(var1);
         if (this.schema != null) {
            this.baseType.bind(this.schema, this);
         }

         this.baseType.addChangeListener(this.changerBase);
      } else {
         this.baseType = null;
      }

   }

   public String getBaseType() {
      return this.baseType != null ? this.baseType.getName() : "";
   }

   public boolean isPE() {
      return false;
   }

   public boolean isXMLPrimitive() {
      return this.getSource().equals(DataTypeIntf.ID) || this.getSource().equals(DataTypeIntf.IDREF) || this.getSource().equals(DataTypeIntf.STRING) || this.getSource().equals(DataTypeIntf.IDREFS) || this.getSource().equals(DataTypeIntf.ENTITY) || this.getSource().equals(DataTypeIntf.ENTITIES) || this.getSource().equals(DataTypeIntf.NMTOKEN) || this.getSource().equals(DataTypeIntf.NMTOKENS) || this.getSource().equals(DataTypeIntf.NOTATION) || this.getSource().equals(DataTypeIntf.ENUMERATION);
   }

   public boolean hasEnumeration() {
      return this.facets.containsKey(Facet.FACET_ENUM_NAME);
   }

   public boolean hasFacet(String var1) {
      return this.facets.containsKey(var1);
   }

   public Facet getFacet(String var1) {
      return this.facets.containsKey(var1) ? (Facet)this.facets.get(var1) : null;
   }

   public boolean enumContains(String var1) {
      if (!this.facets.containsKey(Facet.FACET_ENUM_NAME)) {
         return false;
      } else {
         Facet var2 = (Facet)this.facets.get(Facet.FACET_ENUM_NAME);
         Vector var3 = (Vector)var2.getValue();
         if (var3 != null) {
            for(int var4 = 0; var4 < var3.size(); ++var4) {
               if (((String)var3.elementAt(var4)).equals(var1)) {
                  return true;
               }
            }
         }

         return false;
      }
   }

   public DataTypeIntf applyFacet(Facet var1) {
      DataType var2 = null;

      try {
         boolean var3 = DataTypeFactory.isKnown(this);
         if (var3) {
            var2 = (DataType)this.clearFacets();
            var2.setBaseType(this.type.getName());
            var2.setTypeName((String)null);
         } else {
            var2 = (DataType)this.clone();
         }
      } catch (CloneNotSupportedException var5) {
         Debug.assert(false, "Problem cloning DataType.");
         return this;
      }

      this.applyFacet(var2, var1);
      return var2;
   }

   protected void applyFacet(DataType var1, Facet var2) {
      if (var1.facets.containsKey(var2.getName())) {
         var1.facets.remove(var2.getName());
      }

      var1.facets.put(var2.getName(), var2);
   }

   public DataTypeIntf clearFacets() {
      Object var1 = null;

      try {
         var5 = (DataType)this.clone();
      } catch (CloneNotSupportedException var4) {
         Debug.assert(false, "Problem cloning DataType.");
         return this;
      }

      this.clearFacets(var5);
      return var5;
   }

   protected void clearFacets(DataType var1) {
      var1.facets.clear();
   }

   public DataTypeIntf clearFacet(String var1) {
      Object var2 = null;

      try {
         var6 = (DataType)this.clone();
      } catch (CloneNotSupportedException var5) {
         Debug.assert(false, "Problem cloning DataType.");
         return this;
      }

      this.clearFacet(var6, var1);
      return var6;
   }

   protected void clearFacet(DataType var1, String var2) {
      if (var1.facets.containsKey(var2)) {
         var1.facets.remove(var2);
      }

   }

   public boolean isNotation() {
      return this.getSource().equals(DataTypeIntf.NOTATION);
   }

   public boolean isEnumeration() {
      return this.getSource().equals(DataTypeIntf.ENUMERATION);
   }

   public boolean isNmTokens() {
      return this.getSource().equals(DataTypeIntf.NMTOKENS);
   }

   public String getSource() {
      if (this.type != null && this.type.getName().length() > 0) {
         return this.type.getName();
      } else {
         return this.baseType != null && this.baseType.getName() != null && this.baseType.getName().length() > 0 ? this.baseType.getName() : "";
      }
   }

   public String getDisplay() {
      Debug.assert(false, "getDisplay() in DataType deprecated except for DataTypePE!");
      return this.getSource();
   }

   public String toString() {
      return this.getSource();
   }

   public boolean equals(int var1) {
      return this.equals(DataTypeFactory.createDataType(var1));
   }

   public boolean equalsSource(String var1) {
      return this.getSource().equals(var1);
   }

   public int hashCode() {
      return this.getSource().hashCode();
   }

   public boolean equals(DataType var1) {
      if (var1 == null) {
         return false;
      } else if (this == var1) {
         return true;
      } else if (!DataTypeFactory.isKnown(this) && !DataTypeFactory.isKnown(var1)) {
         if (this.hasFacet(Facet.FACET_ENUM_NAME) != var1.hasFacet(Facet.FACET_ENUM_NAME)) {
            return false;
         } else {
            if (this.hasFacet(Facet.FACET_ENUM_NAME) && var1.hasFacet(Facet.FACET_ENUM_NAME)) {
               Facet var2 = this.getFacet(Facet.FACET_ENUM_NAME);
               Facet var3 = var1.getFacet(Facet.FACET_ENUM_NAME);
               if (var2 != null && var3 != null) {
                  Vector var4 = (Vector)var2.getValue();
                  Vector var5 = (Vector)var3.getValue();
                  if (var4.size() != var5.size()) {
                     return false;
                  }

                  for(int var6 = 0; var6 < var4.size(); ++var6) {
                     if (!((String)var4.elementAt(var6)).equals((String)var5.elementAt(var6))) {
                        return false;
                     }
                  }
               }
            }

            return this.getSource().equals(var1.getSource());
         }
      } else {
         return false;
      }
   }

   public int compareTo(DataType var1) {
      if (var1 == null) {
         return 1;
      } else {
         return this.equals(var1) ? 0 : this.getSource().compareTo(var1.getSource());
      }
   }

   private DataTypeIntf setOptions(DataType var1, Hashtable var2) throws ParserException {
      try {
         Enumeration var3 = var2.keys();

         while(var3.hasMoreElements()) {
            String var4 = (String)var3.nextElement();
            String var5 = (String)var2.get(var4);
            if (var4.equals("decimals")) {
               this.applyFacet(var1, new Facet(Facet.FACET_DECIMALS_NAME, var5));
            } else if (var4.equals("digits")) {
               this.applyFacet(var1, new Facet(Facet.FACET_DIGITS_NAME, var5));
            } else if (var4.equals("maxlength")) {
               this.applyFacet(var1, new Facet(Facet.FACET_MAXLENGTH_NAME, var5));
            } else if (var4.equals("minexclusive")) {
               this.applyFacet(var1, new Facet(Facet.FACET_MINEX_NAME, var5));
            } else if (var4.equals("maxexclusive")) {
               this.applyFacet(var1, new Facet(Facet.FACET_MAXEX_NAME, var5));
            } else if (var4.equals("minvalue")) {
               this.applyFacet(var1, new Facet(Facet.FACET_MIN_NAME, var5));
            } else if (var4.equals("maxvalue")) {
               this.applyFacet(var1, new Facet(Facet.FACET_MAX_NAME, var5));
            } else if (var4.equals(Facet.FACET_LENGTH_NAME)) {
               this.applyFacet(var1, new Facet(Facet.FACET_LENGTH_NAME, var5));
            } else if (var4.equals(Facet.FACET_MINLENGTH_NAME)) {
               this.applyFacet(var1, new Facet(Facet.FACET_MINLENGTH_NAME, var5));
            } else if (var4.equals(Facet.FACET_ENCODING_NAME)) {
               this.applyFacet(var1, new Facet(Facet.FACET_ENCODING_NAME, var5));
            } else if (var4.equals(Facet.FACET_PATTERN_NAME)) {
               this.applyFacet(var1, new Facet(Facet.FACET_PATTERN_NAME, var5));
            } else if (var4.equals(Facet.FACET_PERIOD_NAME)) {
               this.applyFacet(var1, new Facet(Facet.FACET_PERIOD_NAME, var5));
            } else {
               if (!var4.equals(Facet.FACET_ENUM_NAME)) {
                  throw new ParserException(999, "attribute/value pairs parse error.");
               }

               Vector var6 = StringUtilities.parseList(var5, '|');
               this.applyFacet(var1, new Facet(Facet.FACET_ENUM_NAME, var6));
            }
         }

         return var1;
      } catch (Exception var7) {
         throw new ParserException(999, "attribute/value pairs parse error.");
      }
   }

   public DataTypeIntf setOptions(String var1) {
      boolean var2 = DataTypeFactory.isKnown(this);
      boolean var3 = false;
      DataType var4 = (DataType)this.clearFacets();
      if (var2) {
         var4.setBaseType(this.type.getName());
         var4.setTypeName((String)null);
      }

      try {
         Hashtable var22 = XMLStringUtilities.getNameValuePairs(var1);
         var4 = (DataType)this.setOptions(var4, var22);
         return var4;
      } catch (Exception var16) {
         if (!this.isEnumeration() && var1.indexOf(124) < 0 && var1.charAt(0) != '%') {
            if (var1.indexOf(46) >= 0) {
               int var18 = var1.indexOf(46);
               int var6 = var1.indexOf(91);
               int var7 = var1.indexOf(40);
               boolean var8 = false;
               if (var6 == -1 && var7 == -1) {
                  var8 = true;
               } else if (var18 != -1 && (var6 != -1 && var18 < var6 || var7 != -1 && var18 < var7)) {
                  var8 = true;
               }

               if (var8) {
                  String var11 = null;
                  String var12 = null;

                  int var9;
                  for(var9 = var18 - 1; var9 >= 0 && (DTDParser.isDigitChar(var1.charAt(var9)) || var1.charAt(var9) == '*'); --var9) {
                  }

                  ++var9;

                  int var10;
                  for(var10 = var18 + 1; var10 < var1.length() && (DTDParser.isDigitChar(var1.charAt(var10)) || var1.charAt(var10) == '*'); ++var10) {
                  }

                  String var13 = var1.substring(var9, var18);
                  if (!var13.equals("*")) {
                     var11 = var13;
                  }

                  String var14 = var1.substring(var18 + 1, var10);
                  if (!var14.equals("*")) {
                     var12 = var14;
                  }

                  if (var11 != null) {
                     this.applyFacet(var4, new Facet(Facet.FACET_DIGITS_NAME, var11));
                     var3 = true;
                  }

                  if (var12 != null) {
                     this.applyFacet(var4, new Facet(Facet.FACET_DECIMALS_NAME, var12));
                     var3 = true;
                  }
               }
            }

            if (var1.indexOf(40) < 0 && var1.indexOf(91) < 0) {
               if (var3) {
                  return var4;
               } else {
                  try {
                     int var21 = Integer.parseInt(var1);
                     this.applyFacet(var4, new Facet(Facet.FACET_MAXLENGTH_NAME, var1.trim()));
                     return var4;
                  } catch (Exception var15) {
                     Vector var20 = StringUtilities.parseList(var1, '|');
                     this.applyFacet(var4, new Facet(Facet.FACET_ENUM_NAME, var20));
                     return var4;
                  }
               }
            } else {
               int var19 = var1.indexOf(44);

               int var23;
               for(var23 = var19 - 1; var23 >= 0 && this.isValidNumChar(var1.charAt(var23)); --var23) {
               }

               ++var23;

               int var25;
               for(var25 = var19 + 1; var25 < var1.length() && this.isValidNumChar(var1.charAt(var25)); ++var25) {
               }

               boolean var29 = var23 > 0 && var1.charAt(var23 - 1) == '(';
               boolean var30 = var25 < var1.length() && var1.charAt(var25) == ')';
               String var26 = var1.substring(var23, var19);
               if (var26.equals("*")) {
                  var26 = null;
               }

               String var28 = var1.substring(var19 + 1, var25).trim();
               if (var28.equals("*")) {
                  var28 = null;
               }

               if (var26 != null) {
                  this.applyFacet(var4, new Facet(Facet.FACET_MIN_NAME, var26));
               }

               if (var28 != null) {
                  this.applyFacet(var4, new Facet(Facet.FACET_MAX_NAME, var28));
               }

               if (var29) {
                  this.applyFacet(var4, new Facet(Facet.FACET_MINEX_NAME, "true"));
               }

               if (var30) {
                  this.applyFacet(var4, new Facet(Facet.FACET_MAXEX_NAME, "true"));
               }

               return var4;
            }
         } else {
            Vector var5 = StringUtilities.parseList(var1, '|');
            this.applyFacet(var4, new Facet(Facet.FACET_ENUM_NAME, var5));
            return var4;
         }
      }
   }

   private boolean isValidNumChar(char var1) {
      return DTDParser.isDigitChar(var1) || var1 == '.' || DTDParser.isWhiteChar(var1) || var1 == '*' || var1 == '-';
   }

   public String getOptions() {
      boolean var1 = DataTypeFactory.isKnown(this);
      if (var1) {
         return "";
      } else {
         Facet var2 = this.getFacet(Facet.FACET_ENUM_NAME);
         if (var2 != null) {
            Vector var17 = (Vector)var2.getValue();
            Object var18 = null;
            String var19;
            if (var17.size() == 1 && !this.isEnumeration()) {
               var19 = (String)var17.elementAt(0);
               var19 = String.valueOf(String.valueOf("enum=\"").concat(String.valueOf(var19))).concat(String.valueOf("\""));
            } else {
               var19 = StringUtilities.concat(var17.elements(), "", " | ", "");
            }

            return var19;
         } else {
            String var3 = null;
            String var4 = null;
            String var5 = null;
            Facet var6 = this.getFacet(Facet.FACET_DECIMALS_NAME);
            Facet var7 = this.getFacet(Facet.FACET_DIGITS_NAME);
            if (var6 != null) {
               var4 = (String)var6.getValue();
            }

            if (var7 != null) {
               var5 = (String)var7.getValue();
            }

            if (var5 != null || var4 != null) {
               if (var5 != null && var5.toString().equals("0") && var4 != null) {
                  var3 = String.valueOf("*.").concat(String.valueOf(var4 != null ? var4.toString() : "*"));
               } else {
                  var3 = String.valueOf(String.valueOf(var5 != null ? var5.toString() : "*").concat(String.valueOf('.'))).concat(String.valueOf(var4 != null ? var4.toString() : "*"));
               }
            }

            Facet var8 = this.getFacet(Facet.FACET_MAX_NAME);
            Facet var9 = this.getFacet(Facet.FACET_MIN_NAME);
            Facet var10 = this.getFacet(Facet.FACET_MAXEX_NAME);
            Facet var11 = this.getFacet(Facet.FACET_MINEX_NAME);
            if (var8 != null || var9 != null || var3 != null && (var10 != null || var11 != null)) {
               Object var26 = null;
               Object var13 = null;
               String var27;
               if (var9 != null) {
                  var27 = (String)var9.getValue();
               } else {
                  var27 = "*";
               }

               String var28;
               if (var8 != null) {
                  var28 = (String)var8.getValue();
               } else {
                  var28 = "*";
               }

               boolean var14 = false;
               boolean var15 = false;
               if (var10 != null && ((String)var10.getValue()).equals("true")) {
                  var15 = true;
               }

               if (var11 != null && ((String)var11.getValue()).equals("true")) {
                  var14 = true;
               }

               String var16 = String.valueOf(String.valueOf(String.valueOf(String.valueOf((char)(var14 ? '(' : '[')).concat(String.valueOf(var27))).concat(String.valueOf(", "))).concat(String.valueOf(var28))).concat(String.valueOf((char)(var15 ? ')' : ']')));
               return String.valueOf(var3 != null ? String.valueOf(var3).concat(String.valueOf(" ")) : "").concat(String.valueOf(var16));
            } else {
               if (var10 != null) {
                  var3 = String.valueOf(String.valueOf("maxexclusive='").concat(String.valueOf((String)var10.getValue()))).concat(String.valueOf("'"));
               }

               if (var11 != null) {
                  var3 = String.valueOf(String.valueOf(String.valueOf(var3 != null ? String.valueOf(var3).concat(String.valueOf(" ")) : "").concat(String.valueOf("minexclusive='"))).concat(String.valueOf((String)var11.getValue()))).concat(String.valueOf("'"));
               }

               if (var3 != null) {
                  return var3;
               } else {
                  Facet var12 = this.getFacet(Facet.FACET_MAXLENGTH_NAME);
                  if (var12 != null) {
                     return (String)var12.getValue();
                  } else {
                     var12 = this.getFacet(Facet.FACET_LENGTH_NAME);
                     if (var12 != null) {
                        return String.valueOf(String.valueOf(String.valueOf(Facet.FACET_LENGTH_NAME).concat(String.valueOf("='"))).concat(String.valueOf((String)var12.getValue()))).concat(String.valueOf("'"));
                     } else {
                        var12 = this.getFacet(Facet.FACET_MINLENGTH_NAME);
                        if (var12 != null) {
                           return String.valueOf(String.valueOf(String.valueOf(Facet.FACET_MINLENGTH_NAME).concat(String.valueOf("='"))).concat(String.valueOf((String)var12.getValue()))).concat(String.valueOf("'"));
                        } else {
                           var12 = this.getFacet(Facet.FACET_ENCODING_NAME);
                           if (var12 != null) {
                              return String.valueOf(String.valueOf(String.valueOf(Facet.FACET_ENCODING_NAME).concat(String.valueOf("='"))).concat(String.valueOf((String)var12.getValue()))).concat(String.valueOf("'"));
                           } else {
                              var12 = this.getFacet(Facet.FACET_PATTERN_NAME);
                              if (var12 != null) {
                                 return String.valueOf(String.valueOf(String.valueOf(Facet.FACET_PATTERN_NAME).concat(String.valueOf("='"))).concat(String.valueOf((String)var12.getValue()))).concat(String.valueOf("'"));
                              } else {
                                 var12 = this.getFacet(Facet.FACET_PERIOD_NAME);
                                 return var12 != null ? String.valueOf(String.valueOf(String.valueOf(Facet.FACET_PERIOD_NAME).concat(String.valueOf("='"))).concat(String.valueOf((String)var12.getValue()))).concat(String.valueOf("'")) : "";
                              }
                           }
                        }
                     }
                  }
               }
            }
         }
      }
   }

   public void bind(SchemaIntf var1) {
      this.schema = var1;
      if (this.type != null) {
         if (this.type.isBound()) {
            this.type.release();
         }

         this.type.bind(var1, this);
      }

      if (this.baseType != null) {
         if (this.baseType.isBound()) {
            this.baseType.release();
         }

         this.baseType.bind(var1, this);
      }

   }

   public void release() {
      if (this.type != null && this.type.isBound()) {
         this.type.release();
      }

      if (this.baseType != null && this.baseType.isBound()) {
         this.baseType.release();
      }

   }

   public Object clone() throws CloneNotSupportedException {
      DataType var1 = (DataType)super.clone();
      if (this.type != null) {
         var1.setTypeName(this.type.getName());
      }

      if (this.baseType != null) {
         var1.setBaseType(this.baseType.getName());
      }

      var1.facets = (Hashtable)this.facets.clone();
      return var1;
   }

   public void addChangeListener(ChangeListener var1) {
      this.listenerList.add(Class.forName("javax.swing.event.ChangeListener"), var1);
   }

   public void removeChangeListener(ChangeListener var1) {
      this.listenerList.remove(Class.forName("javax.swing.event.ChangeListener"), var1);
   }

   public void fireChangeEvent(String var1) {
      Object[] var2 = this.listenerList.getListenerList();

      for(int var3 = var2.length - 2; var3 >= 0; var3 -= 2) {
         if (var2[var3] == Class.forName("javax.swing.event.ChangeListener")) {
            ((ChangeListener)var2[var3 + 1]).stateChanged((ChangeEvent)null);
         }
      }

   }
}
