package com.extensibility.xml.dt;

import com.extensibility.util.Debug;
import java.util.Hashtable;
import java.util.Vector;

public class DataTypeFactory {
   public static final String[] ALL_TYPES;
   static Hashtable knowns;
   public static final int MS_ACCESS97_DATETIME = 11;

   public static boolean isPrimitive(DataType var0) {
      return var0.isXMLPrimitive();
   }

   public static boolean isKnown(DataType var0) {
      return knowns.get(var0.getTypeName()) != null;
   }

   public static boolean isKnown(String var0) {
      return knowns.get(var0) != null;
   }

   public static DataType createDataTypeFromSQLTypes(short var0) {
      DataType var1;
      switch (var0) {
         case -7:
            var1 = createDataType(DataTypeIntf.BOOLEAN);
            break;
         case -6:
            var1 = createDataType(DataTypeIntf.I1);
            break;
         case -5:
            var1 = createDataType(DataTypeIntf.I8);
            break;
         case -4:
            var1 = createDataType(DataTypeIntf.BINHEX);
            break;
         case -3:
            var1 = createDataType(DataTypeIntf.BINHEX);
            break;
         case -2:
            var1 = createDataType(DataTypeIntf.BINHEX);
            break;
         case -1:
            var1 = createDataType(DataTypeIntf.STRING);
            break;
         case 0:
            var1 = createDataType(DataTypeIntf.STRING);
            break;
         case 1:
            var1 = createDataType(DataTypeIntf.CHAR);
            break;
         case 2:
            var1 = createDataType(DataTypeIntf.NUMBER);
            break;
         case 3:
            var1 = createDataType(DataTypeIntf.FLOAT);
            break;
         case 4:
            var1 = createDataType(DataTypeIntf.I4);
            break;
         case 5:
            var1 = createDataType(DataTypeIntf.I2);
            break;
         case 6:
            var1 = createDataType(DataTypeIntf.FLOAT);
            break;
         case 7:
            var1 = createDataType(DataTypeIntf.R4);
            break;
         case 8:
            var1 = createDataType(DataTypeIntf.R8);
            break;
         case 11:
         case 93:
            var1 = createDataType(DataTypeIntf.DATETIME);
            break;
         case 12:
            var1 = createDataType(DataTypeIntf.STRING);
            break;
         case 91:
            var1 = createDataType(DataTypeIntf.DATE);
            break;
         case 92:
            var1 = createDataType(DataTypeIntf.TIME);
            break;
         case 1111:
            var1 = createDataType(DataTypeIntf.BINHEX);
            break;
         default:
            var1 = createDataType(DataTypeIntf.STRING);
      }

      return var1;
   }

   public static DataType createDataTypeFromClass(Class var0, String var1) {
      if (var0 == Class.forName("java.lang.String")) {
         return createDataType(DataTypeIntf.STRING);
      } else if (var0 == Class.forName("java.lang.Integer")) {
         return createDataType(DataTypeIntf.INT);
      } else if (var0 == Class.forName("java.lang.Boolean")) {
         return createDataType(DataTypeIntf.BOOLEAN);
      } else if (var0 == Class.forName("java.util.Date")) {
         return createDataType(DataTypeIntf.DATETIME);
      } else {
         return var0 == Class.forName("java.lang.Float") ? createDataType(DataTypeIntf.FLOAT) : new DataType(var1, (String)null);
      }
   }

   public static DataType createDataType(String var0) {
      DataType var1 = (DataType)knowns.get(var0);
      return var1 != null ? var1 : new DataType(var0, (String)null);
   }

   public static DataType createDataType(Object var0) {
      if (var0 instanceof Integer) {
         return createDataType((Integer)var0);
      } else if (var0 instanceof String) {
         return new DataTypePE((String)var0, (String)null);
      } else {
         Debug.assert(false, "Unrecognized type object in createDataType");
         return null;
      }
   }

   public static DataType createDataType(int var0) {
      switch (var0) {
         case 1:
            return createDataType(DataTypeIntf.STRING);
         case 2:
            return createDataType(DataTypeIntf.ID);
         case 3:
            return createDataType(DataTypeIntf.IDREF);
         case 4:
            return createDataType(DataTypeIntf.IDREFS);
         case 5:
            return createDataType(DataTypeIntf.ENTITY);
         case 6:
            return createDataType(DataTypeIntf.ENTITIES);
         case 7:
            return createDataType(DataTypeIntf.NMTOKEN);
         case 8:
            return createDataType(DataTypeIntf.NMTOKENS);
         case 9:
            return createDataType(DataTypeIntf.NOTATION);
         case 10:
            return createDataType(DataTypeIntf.ENUMERATION);
         default:
            Debug.assert(false, String.valueOf("Not a known primitive type: ").concat(String.valueOf(var0)));
            return null;
      }
   }

   public static DataTypeEnumeration getTheseTypes(String[] var0) {
      return new DataTypeEnumImpl(var0);
   }

   static {
      ALL_TYPES = new String[]{DataTypeIntf.STRING, DataTypeIntf.ID, DataTypeIntf.IDREF, DataTypeIntf.IDREFS, DataTypeIntf.ENTITY, DataTypeIntf.ENTITIES, DataTypeIntf.NMTOKEN, DataTypeIntf.NMTOKENS, DataTypeIntf.NOTATION, DataTypeIntf.ENUMERATION, DataTypeIntf.NUMBER, DataTypeIntf.INT, DataTypeIntf.DECIMAL, DataTypeIntf.FLOAT, DataTypeIntf.BOOLEAN, DataTypeIntf.DATETIME, DataTypeIntf.DATE, DataTypeIntf.TIME, DataTypeIntf.URI, DataTypeIntf.FIXED144, DataTypeIntf.DATETIMETZ, DataTypeIntf.TIMETZ, DataTypeIntf.INTERVAL, DataTypeIntf.I1, DataTypeIntf.I2, DataTypeIntf.I4, DataTypeIntf.I8, DataTypeIntf.UI1, DataTypeIntf.UI2, DataTypeIntf.UI4, DataTypeIntf.UI8, DataTypeIntf.R4, DataTypeIntf.R8, DataTypeIntf.UUID, DataTypeIntf.BINHEX, DataTypeIntf.BINBASE64, DataTypeIntf.CHAR};
      knowns = new Hashtable();
      knowns.put(DataTypeIntf.ID, new DataType(DataTypeIntf.ID, (String)null));
      knowns.put(DataTypeIntf.IDREF, new DataType(DataTypeIntf.IDREF, (String)null));
      knowns.put(DataTypeIntf.IDREFS, new DataType(DataTypeIntf.IDREFS, (String)null));
      knowns.put(DataTypeIntf.ENTITY, new DataType(DataTypeIntf.ENTITY, (String)null));
      knowns.put(DataTypeIntf.ENTITIES, new DataType(DataTypeIntf.ENTITIES, (String)null));
      knowns.put(DataTypeIntf.NMTOKEN, new DataType(DataTypeIntf.NMTOKEN, (String)null));
      knowns.put(DataTypeIntf.NMTOKENS, new DataType(DataTypeIntf.NMTOKENS, (String)null));
      knowns.put(DataTypeIntf.ENUMERATION, new DataType(DataTypeIntf.ENUMERATION, (String)null));
      knowns.put(DataTypeIntf.NOTATION, new DataType(DataTypeIntf.NOTATION, (String)null));
      knowns.put(DataTypeIntf.STRING, new DataType(DataTypeIntf.STRING, (String)null));
      knowns.put(DataTypeIntf.NUMBER, new DataType(DataTypeIntf.NUMBER, (String)null));
      knowns.put(DataTypeIntf.INT, new DataType(DataTypeIntf.INT, (String)null, (Vector)null));
      knowns.put(DataTypeIntf.BOOLEAN, new DataType(DataTypeIntf.BOOLEAN, (String)null));
      knowns.put(DataTypeIntf.DATETIME, new DataType(DataTypeIntf.DATETIME, (String)null));
      knowns.put(DataTypeIntf.DATETIMETZ, new DataType(DataTypeIntf.DATETIMETZ, (String)null));
      knowns.put(DataTypeIntf.DATE, new DataType(DataTypeIntf.DATE, (String)null));
      knowns.put(DataTypeIntf.TIME, new DataType(DataTypeIntf.TIME, (String)null));
      knowns.put(DataTypeIntf.TIMETZ, new DataType(DataTypeIntf.TIMETZ, (String)null));
      knowns.put(DataTypeIntf.INTERVAL, new DataType(DataTypeIntf.INTERVAL, (String)null));
      knowns.put(DataTypeIntf.I1, new DataType(DataTypeIntf.I1, DataTypeIntf.INT, (Vector)null));
      knowns.put(DataTypeIntf.I2, new DataType(DataTypeIntf.I2, DataTypeIntf.INT, (Vector)null));
      knowns.put(DataTypeIntf.I4, new DataType(DataTypeIntf.I4, DataTypeIntf.INT, (Vector)null));
      knowns.put(DataTypeIntf.I8, new DataType(DataTypeIntf.I8, DataTypeIntf.INT, (Vector)null));
      knowns.put(DataTypeIntf.UI1, new DataType(DataTypeIntf.UI1, DataTypeIntf.INT, (Vector)null));
      knowns.put(DataTypeIntf.UI2, new DataType(DataTypeIntf.UI2, DataTypeIntf.INT, (Vector)null));
      knowns.put(DataTypeIntf.UI4, new DataType(DataTypeIntf.UI4, DataTypeIntf.INT, (Vector)null));
      knowns.put(DataTypeIntf.UI8, new DataType(DataTypeIntf.UI8, DataTypeIntf.INT, (Vector)null));
      knowns.put(DataTypeIntf.R4, new DataType(DataTypeIntf.R4, DataTypeIntf.FLOAT, (Vector)null));
      knowns.put(DataTypeIntf.R8, new DataType(DataTypeIntf.R8, DataTypeIntf.FLOAT, (Vector)null));
      knowns.put(DataTypeIntf.FIXED144, new DataType(DataTypeIntf.FIXED144, DataTypeIntf.DECIMAL));
      knowns.put(DataTypeIntf.UUID, new DataType(DataTypeIntf.UUID, (String)null));
      knowns.put(DataTypeIntf.URI, new DataType(DataTypeIntf.URI, (String)null));
      knowns.put(DataTypeIntf.BINHEX, new DataType(DataTypeIntf.BINHEX, (String)null));
      knowns.put(DataTypeIntf.BINBASE64, new DataType(DataTypeIntf.BINBASE64, (String)null));
      knowns.put(DataTypeIntf.CHAR, new DataType(DataTypeIntf.CHAR, DataTypeIntf.STRING, (Vector)null));
      knowns.put(DataTypeIntf.FLOAT, new DataType(DataTypeIntf.FLOAT, (String)null));
      knowns.put(DataTypeIntf.DECIMAL, new DataType(DataTypeIntf.DECIMAL, (String)null));
   }
}
